@@ -19,12 +19,12 @@ value.
 Except for the base `Schedule` class, some other subclasses of `Schedule`  which may be general used is provided, too. All can
  be easily understand by their names and plot curves.
 """
-from typing import List
+from typing import List, Any
 
 import numpy as np
+from omegaconf import DictKeyType
 
 from lumo.core import BaseParams
-from lumo.core.metaclasses import PropVar
 
 __all__ = ['Cos',
            'Linear',
@@ -60,6 +60,9 @@ class Interpolate(BaseParams):
     def __call__(self, cur):
         raise NotImplementedError()
 
+    def get(self, key: DictKeyType, default_value: Any = None) -> Any:
+        return self(key)
+
     def plot(self, num=1000, left=0, right=1000, show=True):
         """
         Plot a curve of the schedule.
@@ -230,8 +233,6 @@ class Cos(ABCContinuous):
         cos_ratio = 0.5 * (1 + np.cos(ratio * np.pi))
         return start * cos_ratio + end * (1 - cos_ratio)
 
-    def get(self, cur):
-        return self(cur)
 
 
 class Linear(ABCContinuous):
