@@ -2,7 +2,6 @@ import warnings
 from numbers import Number
 
 from . import Attr
-from .metaclasses import PropVar
 from .meter import Meter
 import torch
 import numpy as np
@@ -34,8 +33,9 @@ def wrap_result(metric: MetricType) -> Meter:
     return Meter()
 
 
-class Record(metaclass=PropVar):
+class Record:
     def __init__(self, window_size=500, **kwargs):
+        self._prop = {}
         self._prop.update(kwargs)
         self._cache = []
         self._agg = OrderedDict()  # type:Dict[str,AggItem]
