@@ -5,14 +5,17 @@ import time
 import traceback
 from pathlib import Path
 from typing import Union
-from lumo.utils.logger import Logger
+
 from lumo.decorators.process import call_on_main_process_wrap
-from lumo.proc.path import blobroot, exproot, cache_dir, libhome
+from lumo.proc import glob
+from lumo.proc.dist import is_dist, is_main, local_rank
+from lumo.proc.path import blobroot, libhome, progressroot
+from lumo.proc.path import exproot, local_dir
+from lumo.utils import safe_io as io
+from lumo.utils.fmt import can_be_filename
+from lumo.utils.logger import Logger
 from .base import ExpHook
-from ..proc.dist import is_dist, is_main, local_rank
-from ..proc.path import exproot, local_dir
-from ..utils import safe_io as io
-from ..utils.fmt import can_be_filename
+from ..proc.pid import pid_hash, runtime_pid_obj
 
 
 def checkdir(path: Union[Path, str]):
@@ -27,9 +30,19 @@ class Experiment:
     """
     (by default), the directory structure is as following:
     .lumo (libroot)
+        - progress
+            - ".{pid}" -> hash
+                if pid exists and hash(psutil.Process) == hash in file: is run
+                else: is closed
         - experiments # (exp_root) record information (e.g., .log, params files, etc.)
             - {experiment-name-1}
                 - {test-1}
+                    # infomation
+                    {
+                        progress
+                        pid_hash (for lumo.client monitor)
+                        other_info: git, file, version_lock, etc.
+                    }
                 - {test-2}
             - {experiment-name-2}
                 - {test-1}
@@ -115,6 +128,9 @@ class Experiment:
         return self._test_name
 
     def _create_test_name(self):
+        """
+        [0-9]{6}.[0-9]{3}.[a-z0-9]{3}t
+        """
         from lumo.proc.date import timehash
         from ..utils.fmt import strftime
         fs = os.listdir(self.exp_root)
@@ -142,11 +158,22 @@ class Experiment:
         val = Path(blobroot()).joinpath(self.exp_name, self.test_name)
         return checkdir(val)
 
+    @property
+    def progress_branch(self):
+        val = Path(progressroot())
+        return checkdir(val)
+
     @property
     def test_branch(self):
         val = self.exp_branch.joinpath(self.test_name)
         return checkdir(val)
 
+    def dump_progress(self, ratio: float, update_from=None):
+        res = {'ratio': ratio}
+        if update_from is None:
+            res['update_from'] = update_from
+        self.dump_info('progress', res, append=True)
+
     def dump_info(self, key: str, info: dict, append=False, info_dir='info', set_prop=True):
         fn = self.test_file(f'{key}.json', info_dir)
         if append:
@@ -245,6 +272,9 @@ class Experiment:
         parent = self.blob_branch.joinpath(*args)
         return checkdir(parent).joinpath(filename).as_posix()
 
+    def progress_file(self, filename):
+        return self.progress_branch.joinpath(filename).as_posix()
+
     def blob_dir(self, *args):
         """
 
@@ -284,6 +314,7 @@ class Experiment:
     @call_on_main_process_wrap
     def initial(self):
         self.add_tag(self.__class__.__name__, 'exp_type')
+        self.dump_progress(0)
         self.dump_info('execute', {
             'repo': self.project_root,
             'cwd': os.getcwd(),
@@ -291,6 +322,14 @@ class Experiment:
             'exec_bin': sys.executable,
             'exec_argv': sys.argv
         })
+        self.dump_info('pinfo', {
+            'pid': os.getpid(),
+            'hash': pid_hash(),
+            'obj': runtime_pid_obj(),
+        })
+
+        # register progress
+        io.dump_text(self.test_root, self.progress_file(f'{os.getpid()}'))
 
     @call_on_main_process_wrap
     def start(self):
@@ -308,6 +347,7 @@ class Experiment:
             return
         if self.get_prop('end', False):
             return
+        self.dump_progress(1)
         self.set_prop('end', True)
         for hook in self._hooks.values():  # type: ExpHook
             hook.on_end(self, end_code=end_code, *args, **extra)
@@ -377,6 +417,12 @@ class Experiment:
     @call_on_main_process_wrap
     def set_hook(self, hook: ExpHook):
         hook.regist(self)
+        if not glob.get(hook.config_name, True):
+            self.dump_info(hook.name, {
+                'code': -1,
+                'msg': f'{hook.name} disabled'
+            })
+            return self
         self.logger.info(f'Register {hook}.')
         self._hooks[hook.__class__.__name__] = hook
         self.add_tag(hook.__class__.__name__, 'hooks')
@@ -412,6 +458,12 @@ class Experiment:
         except:
             return []
 
+    def __repr__(self):
+        return f'{self.exp_name}->({self.test_name})'
+
+    def __str__(self):
+        return self.__repr__()
+
 
 class SimpleExperiment(Experiment):
 
