@@ -1,12 +1,15 @@
-import os
 import json
+import os
+
+__all__ = ['debug_mode', 'glob', 'global_config_path', 'local_config_path']
+
+import tempfile
+from typing import overload
 
 GLOBAL_DEFAULT = {
     'home': os.path.expanduser("~/.lumo/"),
     'cache_dir': os.path.expanduser("~/.cache/lumo"),
-    'exp_root': os.path.expanduser("~/.lumo/experiments"),
-    'blob_root': os.path.expanduser("~/.lumo/blob"),
-    'metric_root': os.path.expanduser("~/.lumo/metric"),
+    'dev_branch': 'lumo_experiments',
 }
 
 
@@ -40,7 +43,7 @@ def get_config(path, default):
     return default
 
 
-def create_runtime_config():
+def get_runtime_config():
     glob_cfg = get_config(global_config_path(), GLOBAL_DEFAULT)
     local_cfg = get_config(local_config_path(), {})
     cfg = GLOBAL_DEFAULT
@@ -49,14 +52,21 @@ def create_runtime_config():
     return cfg
 
 
-def pretain_model_path(name):
-    pass
+def debug_mode(base_dir=None, disable_git=True):
+    glob['exp_root'] = tempfile.mkdtemp(dir=base_dir)
+    glob['progress_root'] = tempfile.mkdtemp(dir=base_dir)
+    glob['home'] = tempfile.mkdtemp(dir=base_dir)
+    glob['cache_dir'] = tempfile.mkdtemp(dir=base_dir)
+    glob['blob_root'] = tempfile.mkdtemp(dir=base_dir)
+    glob['metric_root'] = tempfile.mkdtemp(dir=base_dir)
+    glob['HOOK_LOCKFILE'] = False
+    glob['HOOK_LASTCMD_DIR'] = tempfile.mkdtemp(dir=base_dir)
+    glob['HOOK_RECORDABORT'] = False
+    glob['HOOK_TIMEMONITOR'] = False
+
+    if disable_git:
+        glob['HOOK_GITCOMMIT'] = False
 
 
-glob = create_runtime_config()
-
-HOME = glob.get("home")
-CACHE_ROOT = glob.get('cache_dir')
-EXP_ROOT = glob.get('exp_root')
-BLOB_ROOT = glob.get('blob_root')
-METRIC_ROOT = glob.get('metric_root')
+# A dict object contains runtime configuration.
+glob = get_runtime_config()
