@@ -7,6 +7,8 @@ from psutil import __version__ as psutil_version
 
 from lumo import __version__ as lumo_version
 
+__all__ = ['get_lock']
+
 
 class Version:
     # lumo = lumo_version
@@ -17,6 +19,16 @@ class Version:
 
 
 def get_lock(*others):
+    """
+    Used to record the specific version of the run-time dependencies to ensure reproducibility.
+
+    Args:
+        *others: other library to be recorded.
+
+    Returns:
+        A dict instance with each library as the key and its version as the value.
+
+    """
     res = {}
     res['lumo'] = lumo_version
     res.update({k: v for k, v in Version.__dict__.items() if not k.startswith('__')})
