@@ -1,4 +1,5 @@
 import bisect
+import os
 import sys
 import warnings
 from datetime import datetime
@@ -7,29 +8,32 @@ from typing import Union, Dict, Any, Optional, Sequence, Mapping, Callable
 
 import numpy as np
 import torch
-from lumo.contrib.accelerate import Accelerator
-from lumo.contrib.accelerate.utils import send_to_device
-# overwrite send_to_device to resolve https://github.com/pytorch/pytorch/issues/83015
-# from accelerate import Accelerator
-# from accelerate.utils import send_to_device
-
 from accelerate import DistributedDataParallelKwargs
+from accelerate.data_loader import DataLoaderDispatcher, DataLoaderShard
 from torch import nn
 from torch.optim import Optimizer
 from torch.utils.data import DataLoader
 
-from lumo.core import ParamsType, TrainStage, Record, MetricType, Meter, Attr
+from lumo.contrib.accelerate import Accelerator
+from lumo.contrib.accelerate.utils import send_to_device
+from lumo.core import TrainStage, Record, MetricType, Meter
 from lumo.core.disk import TableRow, Metrics
 from lumo.data import DataModule
-from lumo.data.accelerator import DataLoaderShard, DataLoaderDispatcher
 from lumo.data.loader import DataLoaderType, DataLoaderSide
 from lumo.proc import dist
+from lumo.proc import glob
 from lumo.trainer.rnd import RndManager
 from lumo.utils.logger import Logger
+from lumo.utils.fmt import strftime
 from .base import _BaseTrainer
-from .components import TrainerExperiment
+from .components import TrainerExperiment, TrainerParams
 from .saver import Saver
 
+# overwrite send_to_device to resolve https://github.com/pytorch/pytorch/issues/83015
+# from accelerate import Accelerator
+# from accelerate.utils import send_to_device
+ParamsType = TrainerParams
+
 
 class Trainer(_BaseTrainer):
     """
@@ -54,9 +58,11 @@ class Trainer(_BaseTrainer):
     """
 
     callback_function = {
-        "save_keypoint", "save_checkpoint", "save_model", "load_state_dict",
-        'ioptims', 'imodels', 'train', 'test', 'evaluate', 'train_epoch', 'train_step',
-        'test_step', 'evaluate_step', 'process_loader', 'regist_dataloader'
+        # "save_checkpoint", "save_model", "load_state_dict",
+        'imodels',
+        'train', 'test', 'evaluate',
+        'train_epoch', 'train_step', 'test_step', 'evaluate_step',
+        'process_loader', 'regist_dataloader'
     }
 
     def __init__(self, params: ParamsType, dm: DataModule = None):
@@ -93,7 +99,7 @@ class Trainer(_BaseTrainer):
             self.accelerate.state.device = torch.device(device)
 
         if dist.is_main():
-            self.params.to_yaml(self.exp.test_file('params.yaml'))
+            self.params.to_yaml(self.exp.params_fn)
 
         self.set_global_steps(0)
         self.set_epoch_idx(0)
@@ -112,7 +118,7 @@ class Trainer(_BaseTrainer):
     @property
     def saver(self) -> Saver:
         if self._saver is None:
-            self._saver = Saver(self.exp.saver_dir)
+            self._saver = Saver(self.exp.state_dict_dir)
         return self._saver
 
     @property
@@ -135,18 +141,13 @@ class Trainer(_BaseTrainer):
     def world_size(self):
         return dist.world_size()
 
-    @property
-    def wandb(self):
-        import wandb
-        wandb.init(project=self.exp.exp_name, name=self.exp.test_name)
-        wandb.config = self.params.to_dict()
-        return wandb
-
     @property
     def logger(self):
         if self._logger is None:
             from lumo.utils.logger import set_global_logger
             self._logger = Logger()
+            self._logger.use_stdout = glob.get('TRAINER_LOGGER_STDIO', True)
+
             set_global_logger(self._logger)
             if self.params.get('debug', False):
                 self._logger.set_verbose(Logger.V_DEBUG)
@@ -154,6 +155,7 @@ class Trainer(_BaseTrainer):
             if self.is_main:
                 fn = self._logger.add_log_dir(self.exp.log_dir)
                 self.exp.dump_info('logger_args', {'log_dir': fn})
+
         return self._logger
 
     @property
@@ -344,6 +346,19 @@ class Trainer(_BaseTrainer):
 
         return loader
 
+    def save_state_dict(self, name='latest.pth', dirpath=None, only_main=True):
+        if not only_main and self.is_dist:
+            pre, ext = os.path.splitext(name)
+            name = f'{pre}-{self.local_rank}{ext}'
+        if dirpath is None:
+
+            fn = self.exp.state_dict_dir
+        else:
+            fn = os.path.join(dirpath, name)
+        torch.save(self.state_dict(), fn)
+        self.wait_for_everyone()
+        return fn
+
     def load_state_dict(self, state_dict: dict):
         _sub = {'models', 'optims', 'other'}
         _missing = []
@@ -394,11 +409,15 @@ class Trainer(_BaseTrainer):
         self.database.update('commit_hex', commit_hex)
         self.database.update_dict(dict(
             test_name=self.exp.test_name,
+            exp_name=self.exp.exp_name,
+            project=self.exp.project_name,
             path=self.exp.test_root,
             start=datetime.now()))
         self.database.set_params(self.params.to_dict())
         self.database.update('command', ' '.join(sys.argv))
-
+        params_hash = self.params.hash()
+        self.database.update('params_hash', params_hash)
+        self.exp.dump_string('params_hash', params_hash)
         self.icallbacks(self.params)
         self.set_property('initial.callbacks', True)
         self.imodels(self.params)
@@ -449,18 +468,28 @@ class Trainer(_BaseTrainer):
             params = self.params
 
         for eidx in range(params.epoch):
+            # update training progress
+            self.exp.dump_train_eidx(eidx, params.epoch)
             self.set_epoch_idx(eidx)
+
+            # train loop
             epoch_record = self.train_epoch(loader, params, limit_global_steps=limit_global_steps)
-            self.set_property('record', epoch_record)
-            self.set_property('record', epoch_record)
+
+            # self.set_property('record', epoch_record)
+
+            # early stop `train_toggle`
             if self.train_toggle:
                 self.set_property('early_stop', 'train toggle')
                 self.train_toggle = False
                 break
+
+            # early stop by `global_steps`
             if limit_global_steps is not None and self.global_steps >= limit_global_steps:
                 self.set_property('early_stop', f'meet limit_global_steps {limit_global_steps}')
                 break
 
+        # update when train finished
+        self.exp.end()
         self.database.update_dict(dict(end=datetime.now(), finished=True), flush=True)
         self.database.flush()
         return self._prop
@@ -695,7 +724,7 @@ class Trainer(_BaseTrainer):
         """
         self.accelerate.wait_for_everyone()
 
-    def save_model(self, is_best=False, meta_info: Union[str, dict, Attr] = None):
+    def save_model(self, is_best=False, meta_info: Union[str, dict] = None):
         info = self._build_trainer_meta_info(meta_info)
         val = self.saver.save_model(self.eidx, self.model_state_dict(),
                                     meta_info=info,
@@ -703,7 +732,7 @@ class Trainer(_BaseTrainer):
         self.wait_for_everyone()
         return val
 
-    def _build_trainer_meta_info(self, meta_info: Union[str, dict, Attr] = None):
+    def _build_trainer_meta_info(self, meta_info: Union[str, dict] = None):
         info = dict()
         info['eidx'] = self.eidx
         if meta_info is not None:
