@@ -7,10 +7,16 @@ from typing import Union
 
 import numpy as np
 import torch
+import time
+
+
+def int_time():
+    return int(str(time.time()).split(".")[-1])
 
 
 def hashseed(hashitem: Union[int, str]):
-    assert isinstance(hashitem, (int, str))
+    if not isinstance(hashitem, (int, str)):
+        raise AssertionError()
 
     if isinstance(hashitem, str):
         digest = hashlib.md5(hashitem.encode(encoding='utf-8')).digest()
@@ -29,6 +35,8 @@ def fix_seed(seed=10, cuda=True):
 
     Notes:
         When use dataloader and its num_workers is bigger than one, the final results may can't be the same cased by multithread.
+
+        [2023.02.22] Currently (as MPS support is quite new) there is no way to set the seed for MPS directly.
     """
     random.seed(seed)
     np.random.seed(seed)
